// Created on: 2000-09-08
// Created by: data exchange team
// Copyright (c) 2000-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _XCAFDoc_Centroid_HeaderFile
#define _XCAFDoc_Centroid_HeaderFile

#include <Standard.hxx>

#include <gp_Pnt.hxx>
#include <TDF_Attribute.hxx>
#include <Standard_Boolean.hxx>
#include <Standard_OStream.hxx>
class Standard_GUID;
class TDF_Label;
class TDF_RelocationTable;

//! attribute to store centroid
class XCAFDoc_Centroid : public TDF_Attribute
{

public:
  //! class methods
  //! =============
  Standard_EXPORT XCAFDoc_Centroid();

  Standard_EXPORT static const Standard_GUID& GetID();

  //! Find, or create, a Location attribute and set it's value
  //! the Location attribute is returned.
  //! Location methods
  //! ===============
  Standard_EXPORT static occ::handle<XCAFDoc_Centroid> Set(const TDF_Label& label,
                                                           const gp_Pnt&    pnt);

  Standard_EXPORT void Set(const gp_Pnt& pnt);

  Standard_EXPORT gp_Pnt Get() const;

  //! Returns point as argument
  //! returns false if no such attribute at the <label>
  Standard_EXPORT static bool Get(const TDF_Label& label, gp_Pnt& pnt);

  Standard_EXPORT const Standard_GUID& ID() const override;

  Standard_EXPORT void Restore(const occ::handle<TDF_Attribute>& With) override;

  Standard_EXPORT occ::handle<TDF_Attribute> NewEmpty() const override;

  Standard_EXPORT void Paste(const occ::handle<TDF_Attribute>&       Into,
                             const occ::handle<TDF_RelocationTable>& RT) const override;

  Standard_EXPORT Standard_OStream& Dump(Standard_OStream& anOS) const override;

  //! Dumps the content of me into the stream
  Standard_EXPORT void DumpJson(Standard_OStream& theOStream, int theDepth = -1) const override;

  DEFINE_STANDARD_RTTIEXT(XCAFDoc_Centroid, TDF_Attribute)

private:
  gp_Pnt myCentroid;
};

#endif // _XCAFDoc_Centroid_HeaderFile
