// Created on: 2002-10-31
// Created by: Michael SAZONOV
// Copyright (c) 2002-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef BinDrivers_Marker_HeaderFile
#define BinDrivers_Marker_HeaderFile

// The markers used in binary file

enum BinDrivers_Marker
{
  BinDrivers_ENDATTRLIST = -1, // the end of attributes list
  BinDrivers_ENDLABEL    = -2  // the end of label
};

#endif
