// Created on: 1993-04-07
// Created by: Laurent BUCHARD
// Copyright (c) 1993-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _IntCurveSurface_TransitionOnCurve_HeaderFile
#define _IntCurveSurface_TransitionOnCurve_HeaderFile

//! \ Uo     ^        \ U1     ^
//! \       | n       \       | n
//! Surf  ====\======|===   ====\======|===
//! \     .           \     .
//! \    .            \    .
//! U1  \   .          Uo \   .
//!
//! ( In )            ( Out )
//!
//! \           /
//! \         /
//! \       /
//! \     /
//! Surf =====-----=====
//!
//! ( Tangent )
//! Crb and Surf are C1
enum IntCurveSurface_TransitionOnCurve
{
  IntCurveSurface_Tangent,
  IntCurveSurface_In,
  IntCurveSurface_Out
};

#endif // _IntCurveSurface_TransitionOnCurve_HeaderFile
