// Created on: 1992-06-29
// Created by: Remi GILET
// Copyright (c) 1992-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _Geom2dGcc_Type2_HeaderFile
#define _Geom2dGcc_Type2_HeaderFile

enum Geom2dGcc_Type2
{
  Geom2dGcc_CuCuOnCu,
  Geom2dGcc_CiCuOnCu,
  Geom2dGcc_LiCuOnCu,
  Geom2dGcc_CuPtOnCu,
  Geom2dGcc_CuCuOnLi,
  Geom2dGcc_CiCuOnLi,
  Geom2dGcc_LiCuOnLi,
  Geom2dGcc_CuPtOnLi,
  Geom2dGcc_CuCuOnCi,
  Geom2dGcc_CiCuOnCi,
  Geom2dGcc_LiCuOnCi,
  Geom2dGcc_CuPtOnCi
};

#endif // _Geom2dGcc_Type2_HeaderFile
