// Copyright (c) 2013 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _BRepMesh_FastDiscret_HeaderFile
#define _BRepMesh_FastDiscret_HeaderFile

#include <IMeshTools_Parameters.hxx>

class Standard_DEPRECATED(
  "The class BRepMesh_FastDiscret is deprecated. Please use IMeshTools package")
  BRepMesh_FastDiscret
{
public:
  Standard_DEPRECATED("The class BRepMesh_FastDiscret is deprecated. Please use IMeshTools "
                      "package")
  typedef IMeshTools_Parameters Parameters;
};

#endif
