// Created on: 2001-04-24
// Created by: Christian CAILLET
// Copyright (c) 2001-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _StepShape_PlusMinusTolerance_HeaderFile
#define _StepShape_PlusMinusTolerance_HeaderFile

#include <Standard.hxx>

#include <StepShape_ToleranceMethodDefinition.hxx>
#include <StepShape_DimensionalCharacteristic.hxx>
#include <Standard_Transient.hxx>

//! Added for Dimensional Tolerances
class StepShape_PlusMinusTolerance : public Standard_Transient
{

public:
  Standard_EXPORT StepShape_PlusMinusTolerance();

  Standard_EXPORT void Init(const StepShape_ToleranceMethodDefinition& range,
                            const StepShape_DimensionalCharacteristic& toleranced_dimension);

  Standard_EXPORT StepShape_ToleranceMethodDefinition Range() const;

  Standard_EXPORT void SetRange(const StepShape_ToleranceMethodDefinition& range);

  Standard_EXPORT StepShape_DimensionalCharacteristic TolerancedDimension() const;

  Standard_EXPORT void SetTolerancedDimension(
    const StepShape_DimensionalCharacteristic& toleranced_dimension);

  DEFINE_STANDARD_RTTIEXT(StepShape_PlusMinusTolerance, Standard_Transient)

private:
  StepShape_ToleranceMethodDefinition theRange;
  StepShape_DimensionalCharacteristic theTolerancedDimension;
};

#endif // _StepShape_PlusMinusTolerance_HeaderFile
