// Created on: 1996-12-23
// Created by: Alexander BRIVIN
// Copyright (c) 1996-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _Vrml_SFRotation_HeaderFile
#define _Vrml_SFRotation_HeaderFile

#include <Standard.hxx>
#include <Standard_DefineAlloc.hxx>
#include <Standard_Handle.hxx>

#include <Standard_Real.hxx>

//! defines SFRotation type of VRML field types.
//! The 4 values represent an axis of rotation followed by amount of
//! right-handed rotation about the that axis, in radians.
class Vrml_SFRotation
{
public:
  DEFINE_STANDARD_ALLOC

  Standard_EXPORT Vrml_SFRotation();

  Standard_EXPORT Vrml_SFRotation(const double aRotationX,
                                  const double aRotationY,
                                  const double aRotationZ,
                                  const double anAngle);

  Standard_EXPORT void SetRotationX(const double aRotationX);

  Standard_EXPORT double RotationX() const;

  Standard_EXPORT void SetRotationY(const double aRotationY);

  Standard_EXPORT double RotationY() const;

  Standard_EXPORT void SetRotationZ(const double aRotationZ);

  Standard_EXPORT double RotationZ() const;

  Standard_EXPORT void SetAngle(const double anAngle);

  Standard_EXPORT double Angle() const;

private:
  double myRotationX;
  double myRotationY;
  double myRotationZ;
  double myAngle;
};

#endif // _Vrml_SFRotation_HeaderFile
