// Created on: 1997-11-10
// Created by: Jean Yves LEBEY
// Copyright (c) 1997-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _TopOpeBRep_define_HeaderFile
#define _TopOpeBRep_define_HeaderFile

#include <TopOpeBRepDS_define.hxx>

#define MTPvpic TopOpeBRep_VPointInterClassifier
#define MTPvpii TopOpeBRep_VPointInterIterator
#define MTPvpi TopOpeBRep_VPointInter
#define MTPli TopOpeBRep_LineInter
#define MTPfi TopOpeBRep_FacesIntersector
#define MTPei TopOpeBRep_EdgesIntersector
#define MTPfei TopOpeBRep_FaceEdgeIntersector
#define MTPff TopOpeBRep_FacesFiller
#define MTPef TopOpeBRep_EdgesFiller
#define MTPfef TopOpeBRep_FaceEdgeFiller
#define MTPpgt TopOpeBRep_PointGeomTool
#endif
