// Created on: 1993-08-10
// Created by: Remi LEQUETTE
// Copyright (c) 1993-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _BRep_PointOnCurveOnSurface_HeaderFile
#define _BRep_PointOnCurveOnSurface_HeaderFile

#include <Standard.hxx>

#include <BRep_PointsOnSurface.hxx>
#include <Standard_Real.hxx>
class Geom2d_Curve;
class Geom_Surface;
class TopLoc_Location;

//! Representation by a parameter on a curve on a
//! surface.
class BRep_PointOnCurveOnSurface : public BRep_PointsOnSurface
{

public:
  Standard_EXPORT BRep_PointOnCurveOnSurface(const double                     P,
                                             const occ::handle<Geom2d_Curve>& C,
                                             const occ::handle<Geom_Surface>& S,
                                             const TopLoc_Location&           L);

  //! Returns True
  Standard_EXPORT bool IsPointOnCurveOnSurface() const override;

  Standard_EXPORT bool IsPointOnCurveOnSurface(const occ::handle<Geom2d_Curve>& PC,
                                               const occ::handle<Geom_Surface>& S,
                                               const TopLoc_Location&           L) const override;

  Standard_EXPORT const occ::handle<Geom2d_Curve>& PCurve() const override;

  Standard_EXPORT void PCurve(const occ::handle<Geom2d_Curve>& C) override;

  //! Dumps the content of me into the stream
  Standard_EXPORT void DumpJson(Standard_OStream& theOStream, int theDepth = -1) const override;

  DEFINE_STANDARD_RTTIEXT(BRep_PointOnCurveOnSurface, BRep_PointsOnSurface)

private:
  occ::handle<Geom2d_Curve> myPCurve;
};

#endif // _BRep_PointOnCurveOnSurface_HeaderFile
