// Created on: 1997-10-02
// Created by: Denis PASCAL
// Copyright (c) 1997-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _DPrsStd_HeaderFile
#define _DPrsStd_HeaderFile

#include <Standard.hxx>
#include <Standard_DefineAlloc.hxx>

#include <Draw_Interpretor.hxx>

//! commands for presentation based on AIS
//! ======================================
class DPrsStd
{
public:
  DEFINE_STANDARD_ALLOC

  //! Presentation commands
  //! =====================
  //! to load all sketch commands
  Standard_EXPORT static void AllCommands(Draw_Interpretor& I);

  //! to display....etc... ais presentation
  Standard_EXPORT static void AISPresentationCommands(Draw_Interpretor& I);

  //! to repaint...etc... ais viewer
  Standard_EXPORT static void AISViewerCommands(Draw_Interpretor& I);

  //! set/get position attribute
  Standard_EXPORT static void BasicCommands(Draw_Interpretor& I);

  //! Loads all Draw commands of TKDCAF. Used for plugin.
  Standard_EXPORT static void Factory(Draw_Interpretor& theDI);
};

#endif // _DPrsStd_HeaderFile
