// Created on: 2015-08-06
// Created by: Ilya Novikov
// Copyright (c) 2004-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _XCAFDimTolObjects_DimensionGrade_HeaderFile
#define _XCAFDimTolObjects_DimensionGrade_HeaderFile

//! Defines value of grade
enum XCAFDimTolObjects_DimensionGrade
{
  XCAFDimTolObjects_DimensionGrade_IT01,
  XCAFDimTolObjects_DimensionGrade_IT0,
  XCAFDimTolObjects_DimensionGrade_IT1,
  XCAFDimTolObjects_DimensionGrade_IT2,
  XCAFDimTolObjects_DimensionGrade_IT3,
  XCAFDimTolObjects_DimensionGrade_IT4,
  XCAFDimTolObjects_DimensionGrade_IT5,
  XCAFDimTolObjects_DimensionGrade_IT6,
  XCAFDimTolObjects_DimensionGrade_IT7,
  XCAFDimTolObjects_DimensionGrade_IT8,
  XCAFDimTolObjects_DimensionGrade_IT9,
  XCAFDimTolObjects_DimensionGrade_IT10,
  XCAFDimTolObjects_DimensionGrade_IT11,
  XCAFDimTolObjects_DimensionGrade_IT12,
  XCAFDimTolObjects_DimensionGrade_IT13,
  XCAFDimTolObjects_DimensionGrade_IT14,
  XCAFDimTolObjects_DimensionGrade_IT15,
  XCAFDimTolObjects_DimensionGrade_IT16,
  XCAFDimTolObjects_DimensionGrade_IT17,
  XCAFDimTolObjects_DimensionGrade_IT18
};

#endif // _XCAFDimTolObjects_DimensionGrade_HeaderFile
