// Created on: 2015-07-14
// Created by: Irina KRYLOVA
// Copyright (c) 2015 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _StepDimTol_SimpleDatumReferenceModifier_HeaderFile
#define _StepDimTol_SimpleDatumReferenceModifier_HeaderFile

enum StepDimTol_SimpleDatumReferenceModifier
{
  StepDimTol_SDRMAnyCrossSection,
  StepDimTol_SDRMAnyLongitudinalSection,
  StepDimTol_SDRMBasic,
  StepDimTol_SDRMContactingFeature,
  StepDimTol_SDRMDegreeOfFreedomConstraintU,
  StepDimTol_SDRMDegreeOfFreedomConstraintV,
  StepDimTol_SDRMDegreeOfFreedomConstraintW,
  StepDimTol_SDRMDegreeOfFreedomConstraintX,
  StepDimTol_SDRMDegreeOfFreedomConstraintY,
  StepDimTol_SDRMDegreeOfFreedomConstraintZ,
  StepDimTol_SDRMDistanceVariable,
  StepDimTol_SDRMFreeState,
  StepDimTol_SDRMLeastMaterialRequirement,
  StepDimTol_SDRMLine,
  StepDimTol_SDRMMajorDiameter,
  StepDimTol_SDRMMaximumMaterialRequirement,
  StepDimTol_SDRMMinorDiameter,
  StepDimTol_SDRMOrientation,
  StepDimTol_SDRMPitchDiameter,
  StepDimTol_SDRMPlane,
  StepDimTol_SDRMPoint,
  StepDimTol_SDRMTranslation
};

#endif